#include "communication/UValue.hpp"
#include "communication/UARTCom.hpp"
#include "Arduino.h"
#include "communication/UValue.hpp"
#include "constants.h"

/**
 * @brief sends the current speed of both motors
 * 
 * @param cp component
 * @param value speed of motor
 */
void UValue::sendEncoderValue (U_Component cp, float value)
{
	union {
		float f;
		uint8_t b[4];
	} u;
	u.f = value;
	uint8_t switched_b[4];
	for (int i = 0; i < 4; i++) {
		switched_b[i] = u.b[3 - i];
	}

	UARTCom::send(VALUE, 4, cp, switched_b);
}

/**
 * @brief sends the current speed of both motors
 * 
 * @param speed_right  speed of right motor
 * @param speed_left speed of left motor
 * @param pos_right current integrated position of right rotor
 * @param pos_left current integrated position of left rotor
 */
void UValue::sendBothEncoderValues (float speed_right, float speed_left, float pos_right, float pos_left)
{
	union {
		float f[4];
		uint8_t b[16];
	} u;
	u.f[0] = speed_right;
	u.f[1] = speed_left;
	u.f[2] = pos_right;
	u.f[3] = pos_left;

	uint8_t switched_byte_array[16];
	for (int i = 0; i < 4; i++) {
		switched_byte_array[i] = u.b[3 - i];
		switched_byte_array[i+4] = u.b[7 - i];
		switched_byte_array[i+8] = u.b[11 - i];
		switched_byte_array[i+12] = u.b[15 - i];
	}

	UARTCom::send(VALUE, 16, ALL_MOTORS, switched_byte_array);
}