#include <Arduino.h>
#include "communication/UARTCom.hpp"
#include "communication/UValue.hpp"

long controlTimer = 0;
long uartTransmitTimer = 0;
int speed = 0;

#include <PinChangeInt.h>
#include <PinChangeIntConfig.h>
#include <PID_Beta6.h>
#include <MotorWheel.h>
#ifndef MICROS_PER_SEC
#define MICROS_PER_SEC 1000000
#endif
irqISR(irq1, isr1);
irqISR(irq2, isr2);
MotorWheel wheelLeft(9, 8, 4, 5, &irq1, REDUCTION_RATIO, 300);	// Motor PWM:Pin9, DIR:Pin8, Encoder A:Pin6, B:Pin7
MotorWheel wheelRight(10, 11, 6, 7, &irq2, REDUCTION_RATIO, 300); // Motor PWM:Pin9, DIR:Pin8, Encoder A:Pin6, B:Pin7

void setup(void)
{
	// configure board for highest PWM frequency available
	TCCR1B = TCCR1B & 0xf8 | 0x01; // Pin9,Pin10 PWM 31250Hz, Silent PWM

	// initialize PID for both wheels
	wheelLeft.PIDEnable(0.26,0.02,0.02,10);// Enable PID
	wheelRight.PIDEnable(0.26,0.02,0.02,10);// Enable PID

	// initialize UART communication
	UARTCom::init(UARTCom::data);
	controlTimer = millis();
	uartTransmitTimer = millis();
	delay(100);

	// send reset timestamp request
	// this is currently not used but could be used to synchronize the timestamp in messages
	uint8_t pl[1];
	pl[0] = REQUEST_RESET_TIMESTAMP;
	UARTCom::send(REQUEST, 1, DEFAULT_COMPONENT, pl);
}

void loop(void)
{
	// regulate boh wheels
	wheelLeft.PIDRegulate();
	wheelRight.PIDRegulate();

	// look if any bytes can be received
	UARTCom::read(UARTCom::data);
}